/******************************************************************************************/
/*																						  */
/* File Name:  Ovl.asm									 						  		  */
/*																						  */
/* Date:  August 27, 2002																  */
/*																						  */
/* Purpose:  This file contains the main program.  It has the interrupt vector table	  */
/* 			 setup, with the DMA10 completion interrupt clearing the DMA enable.		  */
/*	 		 		  																	  */
/* 			 In the main program, the "OvlInit" routine is first called to initialize	  */
/* 			 the overlays' sections' info.  Then, the "OvlSecInfo" routine is called	  */
/*			 to parse and perform housekeeping on all the info.	  Finally, the overlays   */
/*			 are called (blinkone, blinktwo, etc.) and the overlay manager is called upon */
/*			 within these overlay calls to DMA the appropriate overlay into internal 	  */
/*			 memory.  And this process repeats.											  */
/*																						  */
/******************************************************************************************/



#include "def21161.h"


//Interrupt Vector Table
.segment/pm mem_rth;
	nop; nop; nop; jump start;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
	nop; nop; nop; nop;
// ============================== EPO_ISR  =================================================
// DMA Channel 10 (EPB0) Complete. This code must resolve to 0x50 in the interrupt vector table.
// If any code is added or removed prior to this, the kernel will not function.

ep0i_at_0x50:   R2=DM(DMAC10);
                RTI (DB);
                R12=r12-r12;
                DM(DMAC10)=R12;          //  zeroed between uses.

// ============================= END  EP0_ISR  =========================================



//Main program
.segment/pm mem_pmco;
.extern _OvlInit;
.extern _OvlSecInfo;
.extern _OverlayManager;

start:

	call _OvlInit;				//find overlay sections in ROM			
	call _OvlSecInfo;			//parse all information gathered in previous call

		
	//Call each overlay.
	//Repeat process in the end.
	
loadoverlays:
	
	call blinkone;
	call blinktwo;
	call blinkthree;
	call blinkfour;				
	call blinkfour;				//Note that since the previous call was to the same overlay,
								//there will not be a second DMA.  The code will check and 
								//realize the "blinkfour" overlay has already been DMA'ed 
								//and will just jump to execution.
	call blinkthree;
	call blinktwo;
	call blinkone;
	
	jump loadoverlays;			//repeats process over and over again
