/******************************************************************************************/
/*																						  */
/* File Name:  Ovl_Init.asm									 						  	  */
/*																						  */	 
/* Date:  August 27, 2002																  */
/*																						  */		
/* Purpose:  This file runs through the ROM to look for where the overlays' sections	  */
/* 			 reside.  It then places the info for each section (live address, count 	  */	
/*	 		 size, and type) into respective buffers.  It also checks the types and 	  */
/* 			 accounts for how many words to increment in the ROM to look for the 		  */
/* 			 next section's info.										 				  */
/*																						  */
/******************************************************************************************/


#include "def21161.h"

.segment/pm mem_pmco;
.global _OvlInit;

_OvlInit:


// Zero out L-registers so I-registers can be used without wrap
                L0=0; L1=0; L2=0; L3=0;		//DAG1
                L4=0; L5=0; L6=0; L7=0;
                
                L8=0; L9=0; L10=0; L11=0;   //DAG2
				L12=0; L13=0; L14=0; L15=0;
                
                //Init M-registers
                M0=1; M1=0; M2=0; M3=0;    	//DAG1
                M4=0; M5=0; M6=1; M7=0;            
				
                M8=0; M9=0; M10=1; M11=0;
				M12=1; M13=0; M14=1; M15=1;		//DAG2
				
				//Init index registers
				I9=total_live_addr;
				I10=total_sec_size;
				I13=total_sec_type;
				I14=num_ovl_sec;
	
                R8 = 2;                 // Used to decrement tag by two in test_dm16_zero
                R14 = 0x421;            // Used in read_prom_word to config and begin DMAC10
                R15 = 6;
                I4 = 0x40004 ;          // DMA destination address 0x40004 used as local scratch

				R0 = 0x04000600;
				DM(EIEP0) = R0;

                R12=DM(SYSCON);         // R12 = hold original SYSCON setting
                R11=BSET R12 BY 1;      // R11 ~ to set BSO bit for reading ROM
                R10=BCLR R12 BY 1;      // R10 ~ to clear BSO bit for ext write
                DM(SYSCON)=R10;         // Clear BSO bit for writing RAM

                DM(IMEP0)=M13;          // EPB0 Internal Modify = 0
                DM(EMEP0)=M14;          // EPB0 External Modify = 1

                BIT SET IMASK 0x8000;   // Enable EP0 interrupt
                BIT SET MODE1 0x1800;   // Enable interrupts and nesting


// ==========================  GET_ADDR  =============================================
// The processor jump-table holds the address of the boot-image for each DSP.
// (The image may also be relocated issuing the -p switch to the loader.)
// For non-MP systems (ID==0), the boot image resides at address 0x62A
// which is the first address after the kernel and processor jump-table.
// This subroutine executes once.
// ---------------------------------------------------------------------------
get_addr:       R0=DM(SYSTAT);
                R0=FEXT R0 BY 8:3;      // Determine this processor ID
                CALL read_prom_word;
                COMP(R0, R2);           // search for this processor's ID in jump table
                IF NE JUMP (PC,-2);  // If ID != fetch another ID keep looking
                R2 = 0x04000000;
				R3 = R2 + R3;
				DM(EIEP0)=R3;           // where this ID's data resides in PROM
// ========================= END GET_ADDR  ===========================================


// ==========================  READ_BOOT_INFO  =======================================
// Places TAG in R0, Internal Count in R2, and Destination Address in R3
// -----------------------------------------------------------------------------------
read_boot_info: CALL read_prom_word;    // read first tag and count
                R0=PASS R2;             // r0 holds tag
                CALL read_prom_word;
				jump check_routine;

// ==========================  read_prom_word  =======================================
// read_prom_word is a callable subroutine that DMA's in one 48-bit word from PROM.
// It places the MS 32-bits in R3 and the LS 16 (right justified) in R2.
// For example, given the 48-bit word 0x112233445566,
//        R2 holds 0x00005566
//        R3 holds 0x11223344
// -----------------------------------------------------------------------------------
read_prom_word: R13=DM(SYSCON);         // Save old value of SYSCON
                DM(SYSCON)=R11;         // Set BSO bit for ROM read
                DM(IIEP0)=I4;           // 0x40004 = DMA destination address
                DM(CEP0)=M14;           // 1 = DMA internal count
                DM(ECEP0)=R15;          // 1 = DMA external count
                DM(DMAC10)=R14;         // 0x421 Start DMA
                IDLE;                   // Wait for DMA to complete
                PX=PM(0x40004);         // Read word from scratch
                DM(SYSCON)=R13;         // Reset SYSCON to previous
                R2=PX1;                 // Copy PX values into DREGS
              RTS (DB);
                    R2=LSHIFT R2 by -16;    // Downshift 16 bits to adjust for the extension of PX for HH
read_prom_word.end: R2=PASS R2, R3=PX2;     // Test the length


// =============================  CHECK_ROUTINE  ==================================
// R0 holds the TAG, R2 holds the Word Count, and R3 holds the Destination Address.
// Overlays have been mapped to a dummy live address in SDRAM space,
// Therefore, check for a destination address of greater than or equal to 0x600000
// in R3.  To advance the count by which to increment readings from the ROM, check
// each tag info and determine whether it needs to be incremented by 12 words, 6
// words, or none.  
// Also check for final init.  When final init tag comes across, then the 
// end of program is coming and there are no more overlays. 
// ---------------------------------------------------------------------------


check_routine:

	//final init
	R8 = 0x0;
	R7 = R0 - R8;
	if eq jump finalinit;



	//If tag equals 0x15:  External memory code
	R8 = 0x15;		
	R7 = R0 - R8;
	if eq jump initliveinfo1;	

	//If tag equals 0x19:  External memory tag for nops
	R8 = 0x19;		
	R7 = R0 - R8;
	if eq jump initliveinfo2;
	
	
	//multiply by 12 if tag equal 0x12 or 0x10
	R8 = 0x12;
	R6 = 0x10;
	R7 = R0 - R8;
	if eq jump multiply_c;
	R7 = R0 - R6;
	if eq jump multiply_c;
	
	//tag equal 0x11 or 0xF
	R8 = 0x11;
	R7 = R0 - R8;
	if eq jump no_inc;
	
	R8 = 0xF;
	R7 = R0 - R8;
	if eq jump no_inc;
	
	//multiply by 6
	R8 = 0x16;
	R6 = 0x12;
	
	//if tag greater than 0x16
	R7 = R0-R8;
	if gt jump no_inc;
	
	//if tag greater than 0x12
	R7 = R0-R6;
	if gt jump multiply_six;


	//if tag greater than 0xa
	R6 = 0xa;	
	R7 = R0-R6;
	if gt jump multiply_six;
	
	

	//if tag greater than 0x6
	R6 = 0x6;
	R7 = R0-R6;
	if gt jump no_inc;
	
	R6 = 0x3;
	R7 = R0-R6;
	if gt jump multiply_six;

	
	jump no_inc;
	
	
	//multiply by six to advance count to next tag
multiply_six:
	R12 = DM(EIEP0);
	R2 = R2*R15 (uui);
	R1 = R12 + R2;
	DM(EIEP0) = R1;
	jump read_boot_info;
	

	
	//multiply by 0xC to advance count to next tag
multiply_c:	
	R8 = 0xc;
	R12 = DM(EIEP0);
	R2 = R2*R8 (uui);
	R1 = R12 + R2;
	DM(EIEP0) = R1;
	jump read_boot_info;

	
		
initliveinfo1:
	R8 = 0x600000;
	R7 = R3-R8;
	if lt jump no_inc;
	R12 = DM(EIEP0);
	PM(I9,M12) = R12;		//write ROM live address to total_live_addr buffer
	PM(I10,M12) = R2;		//write word count to total_sec_size buffer
	PM(I13,M12) = R0;
	
	jump multiply_six;

initliveinfo2:	
	R8 = 0x600000;
	R7 = R3-R8;
	if lt jump no_inc;
	R12 = DM(EIEP0);
	PM(I9,M12) = R12;	//write ROM live address to total_live_addr buffer
	PM(I10,M12) = R2;		//write word count to total_sec_size buffer
	PM(I13,M12) = R0;
	jump no_inc;

//no increment needed
no_inc:
	jump read_boot_info;	
			
finalinit:
	rts;
