/******************************************************************************************/
/*																						  */
/* File Name:  Ovl_Sec_Info.asm									 						  */
/*																						  */	 
/* Date:  August 27, 2002																  */
/*																						  */		
/* Purpose:  This file parses all the info collected in the Ovl_Init.asm file			  */
/* 			 for each overlay.  It checks for the number of	sections within each		  */	
/*	 		 overlay.  It also checks each overlay section's type and size.  It			  */
/* 			 accounts for the overlay id that's included in the data for each			  */
/* 			 overlay by adding 6 words to the live address.  It accounts 				  */
/*			 for the overlay id by subtracting 1 from the count size of the overlay.	  */
/*			 Then, the information is written to the respective address and size buffers. */
/*																						  */
/******************************************************************************************/



#include "def21161.h"

.segment/pm mem_pmco;
.global _OvlSecInfo;

_OvlSecInfo:

	
/* This first section of code checks the number of sections there are within each overlay */
				
				R10 = 0x4;				//count of 4 overlays in this project
				
				I12 = runWordSize;		
				I10 = total_sec_size;
				I13 = num_ovl_sec;
				I9 = total_live_addr;
					
				R5=1;
	check_ovl:	R12=0;					//R12 stores number of sections.  Initiate to 0 to start.
				 
				R9 = PM(I12,M12);		//read the total overlay size
repeat_check:	R8 = PM(I10,M12);		//read the individual section size			
			
				R12 = R12+R5;			//increment the total number of sections by 1
				
				R9 = R9-R8;					//subtract section size from total overlay size
				if ne jump repeat_check;	//if not equal, then there are more sections in this
											//overlay.	repeat the check.			
			    
				PM(I14,M12)=R12;		//if equal, then there are no more sections in this overlay
										//write number of sections in each overlay to buffer
										
			    R10 = R10-R5;			//repeat for next overlay. 
			    if ne jump check_ovl;	//stop when finish checking all 4 overlays in project.
			    
/* The next section of code accounts for the 1 extra address in the total_live_addr buffer */			    
/* holding the address of the overlay id information and the 1 extra word count in the     */
/* total_sec_size buffer.																   */ 
			    
			    I10 = total_sec_size;	//pointer to beginning of section size buffer 
			    R10 = 0x4;				//R10 holds loop counter.  In this case, 4 overlays
			    R4=0x6;			    	//6 8-bit words in ROM to increment count by
remove_id:	    R2 = PM(I13,M12);		//Read number of sections in overlay
			    M14 = R2;				//Set M14 to number of sections in overlay
			    R6 = PM(I10,M9);		//Read individual section size buffer
			    R3 = PM(I9,M9);			//Read live address buffer
			    R6 = R6-R5;				//Subtract the section size by 1 (account for ovl id)
				R3 = R3+R4;				//Add live address by 0x6 (account for ovl id)
				PM(I10,M14)=R6;			//Write back section size to buffer
				PM(I9,M14)=R3;			//Write back live address to buffer
				R10 = R10-R5;			//Do this for all four overlays.  
			    if ne jump remove_id;	//Repeat if not done for all four overlays.
						    			    
			    
	done:		rts;
